@@ -116,6 +116,7 @@ enum
   PROP_STATUS = 1,
   PROP_STATUS_REASON,
   PROP_CONNECTION_READY,
+  PROP_SELF_HANDLE,
   N_PROPS
 };
 
@@ -142,6 +143,9 @@ tp_connection_get_property (GObject *object,
     case PROP_STATUS_REASON:
       g_value_set_uint (value, self->priv->status_reason);
       break;
+    case PROP_SELF_HANDLE:
+      g_value_set_uint (value, self->priv->self_handle);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
       break;
@@ -271,6 +275,61 @@ introspect_contacts (TpConnection *self)
        got_contact_attribute_interfaces, NULL, NULL, NULL);
 }
 
+static void
+_tp_connection_set_self_handle (TpConnection *self,
+                 guint self_handle)
+{
+  if (self_handle != self->priv->self_handle)
+    {
+      self->priv->self_handle = self_handle;
+      g_object_notify ((GObject *) self, "self-handle");
+    }
+}
+
+static void
+got_self_handle (TpConnection *self,
+                 guint self_handle,
+                 const GError *error,
+                 gpointer user_data G_GNUC_UNUSED,
+                 GObject *user_object G_GNUC_UNUSED)
+{
+
+  if (error != NULL)
+    {
+      DEBUG ("%p: GetSelfHandle() failed: %s", self, error->message);
+      self_handle = 0;
+      /* FIXME: abort the readying process */
+    }
+
+  _tp_connection_set_self_handle (self, self_handle);
+  tp_connection_continue_introspection (self);
+}
+
+static void
+on_self_handle_changed (TpConnection *self,
+                        guint self_handle,
+                        gpointer user_data G_GNUC_UNUSED,
+                        GObject *user_object G_GNUC_UNUSED)
+{
+  _tp_connection_set_self_handle (self, self_handle);
+}
+
+static void
+get_self_handle (TpConnection *self)
+{
+  g_assert (self->priv->introspect_needed != NULL);
+
+  tp_cli_connection_connect_to_self_handle_changed (self,
+      on_self_handle_changed, NULL, NULL, NULL, NULL);
+
+  /* GetSelfHandle is deprecated in favour of the SelfHandle property,
+   * but until Connection has other interesting properties, there's no point in
+   * trying to implement a fast path; GetSelfHandle is the only one guaranteed
+   * to work, so we'll sometimes have to call it anyway */
+  tp_cli_connection_call_get_self_handle (self, -1,
+       got_self_handle, NULL, NULL, NULL);
+}
+
 static void
 tp_connection_got_interfaces_cb (TpConnection *self,
                                  const gchar **interfaces,
@@ -278,6 +337,8 @@ tp_connection_got_interfaces_cb (TpConnection *self,
                                  gpointer user_data,
                                  GObject *user_object)
 {
+  TpConnectionProc func;
+
   if (error != NULL)
     {
       DEBUG ("%p: GetInterfaces() failed, assuming no interfaces: %s",
@@ -291,6 +352,9 @@ tp_connection_got_interfaces_cb (TpConnection *self,
   self->priv->introspect_needed = g_array_new (FALSE, FALSE,
       sizeof (TpConnectionProc));
 
+  func = get_self_handle;
+  g_array_append_val (self->priv->introspect_needed, func);
+
   if (interfaces != NULL)
     {
       const gchar **iter;
@@ -306,15 +370,13 @@ tp_connection_got_interfaces_cb (TpConnection *self,
 
               if (q == TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACTS)
                 {
-                  TpConnectionProc func = introspect_contacts;
-
+                  func = introspect_contacts;
                   g_array_append_val (self->priv->introspect_needed, func);
                 }
               else if (q == TP_IFACE_QUARK_CONNECTION_INTERFACE_ALIASING)
                 {
                   /* call GetAliasFlags */
-                  TpConnectionProc func = introspect_aliasing;
-
+                  func = introspect_aliasing;
                   g_array_append_val (self->priv->introspect_needed,
                       func);
                 }
@@ -322,16 +384,14 @@ tp_connection_got_interfaces_cb (TpConnection *self,
               else if (q == TP_IFACE_QUARK_CONNECTION_INTERFACE_AVATARS)
                 {
                   /* call GetAvatarRequirements */
-                  TpConnectionProc func = introspect_avatars;
-
+                  func = introspect_avatars;
                   g_array_append_val (self->priv->introspect_needed,
                       func);
                 }
               else if (q == TP_IFACE_QUARK_CONNECTION_INTERFACE_PRESENCE)
                 {
                   /* call GetStatuses */
-                  TpConnectionProc func = introspect_presence;
-
+                  func = introspect_presence;
                   g_array_append_val (self->priv->introspect_needed,
                       func);
                 }
@@ -371,6 +431,107 @@ tp_connection_status_changed (TpConnection *self,
     }
 }
 
+static void
+tp_connection_connection_error_cb (TpConnection *self,
+                                   const gchar *error_name,
+                                   GHashTable *details,
+                                   gpointer user_data,
+                                   GObject *weak_object)
+{
+  if (self->priv->connection_error != NULL)
+    {
+      g_error_free (self->priv->connection_error);
+      self->priv->connection_error = NULL;
+    }
+
+  tp_proxy_dbus_error_to_gerror (self, error_name,
+        tp_asv_get_string (details, "debug-message"),
+        &(self->priv->connection_error));
+}
+
+static void
+tp_connection_status_reason_to_gerror (TpConnectionStatusReason reason,
+                                       GError **error)
+{
+  TpError code;
+  const gchar *message;
+
+  switch (reason)
+    {
+    case TP_CONNECTION_STATUS_REASON_NONE_SPECIFIED:
+      code = TP_ERROR_DISCONNECTED;
+      message = "Disconnected for unspecified reason";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_REQUESTED:
+      code = TP_ERROR_CANCELLED;
+      message = "User requested disconnection";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_NETWORK_ERROR:
+      code = TP_ERROR_NETWORK_ERROR;
+      message = "Network error";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_ENCRYPTION_ERROR:
+      code = TP_ERROR_ENCRYPTION_ERROR;
+      message = "Encryption error";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_NAME_IN_USE:
+      code = TP_ERROR_NOT_YOURS;
+      message = "Name in use";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_NOT_PROVIDED:
+      code = TP_ERROR_CERT_NOT_PROVIDED;
+      message = "Server certificate not provided";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_UNTRUSTED:
+      code = TP_ERROR_CERT_UNTRUSTED;
+      message = "Server certificate CA not trusted";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_EXPIRED:
+      code = TP_ERROR_CERT_EXPIRED;
+      message = "Server certificate expired";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_NOT_ACTIVATED:
+      code = TP_ERROR_CERT_NOT_ACTIVATED;
+      message = "Server certificate not valid yet";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_HOSTNAME_MISMATCH:
+      code = TP_ERROR_CERT_HOSTNAME_MISMATCH;
+      message = "Server certificate has wrong hostname";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_FINGERPRINT_MISMATCH:
+      code = TP_ERROR_CERT_FINGERPRINT_MISMATCH;
+      message = "Server certificate fingerprint mismatch";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_SELF_SIGNED:
+      code = TP_ERROR_CERT_SELF_SIGNED;
+      message = "Server certificate is self-signed";
+      break;
+
+    case TP_CONNECTION_STATUS_REASON_CERT_OTHER_ERROR:
+      code = TP_ERROR_CERT_INVALID;
+      message = "Unspecified server certificate error";
+      break;
+
+    default:
+      g_set_error (error, TP_ERRORS_DISCONNECTED, reason,
+          "Unknown disconnection reason");
+      return;
+    }
+
+  g_set_error (error, TP_ERRORS, code, "%s", message);
+}
+
 static void
 tp_connection_status_changed_cb (TpConnection *self,
                                  guint status,
@@ -392,11 +553,16 @@ tp_connection_status_changed_cb (TpConnection *self,
 
   if (status == TP_CONNECTION_STATUS_DISCONNECTED)
     {
-      GError *error = g_error_new (TP_ERRORS_DISCONNECTED, reason,
-          "Disconnected: reason %d", reason);
+      if (self->priv->connection_error == NULL)
+        {
+          tp_connection_status_reason_to_gerror (reason,
+              &(self->priv->connection_error));
+        }
 
-      tp_proxy_invalidate ((TpProxy *) self, error);
-      g_error_free (error);
+      tp_proxy_invalidate ((TpProxy *) self, self->priv->connection_error);
+
+      g_error_free (self->priv->connection_error);
+      self->priv->connection_error = NULL;
     }
 }
 
@@ -423,6 +589,13 @@ tp_connection_got_status_cb (TpConnection *self,
     }
 }
 
+static void
+tp_connection_invalidated (TpConnection *self)
+{
+  _tp_connection_set_self_handle (self, 0);
+  _tp_connection_clean_up_handle_refs (self);
+}
+
 static GObject *
 tp_connection_constructor (GType type,
                            guint n_params,
@@ -435,9 +608,11 @@ tp_connection_constructor (GType type,
   /* Connect to my own StatusChanged signal.
    * The connection hasn't had a chance to become invalid yet, so we can
    * assume that this signal connection will work */
-  DEBUG ("Connecting to StatusChanged");
+  DEBUG ("Connecting to StatusChanged and ConnectionError");
   tp_cli_connection_connect_to_status_changed (self,
       tp_connection_status_changed_cb, NULL, NULL, NULL, NULL);
+  tp_cli_connection_connect_to_connection_error (self,
+      tp_connection_connection_error_cb, NULL, NULL, NULL, NULL);
 
   /* get my initial status */
   DEBUG ("Calling GetStatus");
@@ -447,7 +622,7 @@ tp_connection_constructor (GType type,
   _tp_connection_init_handle_refs (self);
 
   g_signal_connect (self, "invalidated",
-      G_CALLBACK (_tp_connection_clean_up_handle_refs), NULL);
+      G_CALLBACK (tp_connection_invalidated), NULL);
 
   DEBUG ("Returning %p", self);
   return (GObject *) self;
@@ -486,6 +661,12 @@ tp_connection_finalize (GObject *object)
       self->priv->contact_attribute_interfaces = NULL;
     }
 
+  if (self->priv->connection_error != NULL)
+    {
+      g_error_free (self->priv->connection_error);
+      self->priv->connection_error = NULL;
+    }
+
   ((GObjectClass *) tp_connection_parent_class)->finalize (object);
 }
 
@@ -555,6 +736,19 @@ tp_connection_class_init (TpConnectionClass *klass)
   g_object_class_install_property (object_class, PROP_STATUS,
       param_spec);
 
+  /**
+   * TpConnection:self-handle:
+   *
+   * The %TP_HANDLE_TYPE_CONTACT handle of the local user on this connection,
+   * or 0 if we don't know yet or if the connection has become invalid.
+   */
+  param_spec = g_param_spec_uint ("self-handle", "Self handle",
+      "The local user's Contact handle on this connection", 0, G_MAXUINT32,
+      0,
+      G_PARAM_READABLE | G_PARAM_STATIC_STRINGS);
+  g_object_class_install_property (object_class, PROP_SELF_HANDLE,
+      param_spec);
+
   /**
    * TpConnection:status-reason:
    *
@@ -668,6 +862,32 @@ finally:
   return ret;
 }
 
+/**
+ * tp_connection_get_self_handle:
+ * @self: a connection
+ *
+ * Return the %TP_HANDLE_TYPE_CONTACT handle of the local user on this
+ * connection, or 0 if the connection is not ready (the
+ * TpConnection:connection-ready property is false) or has become invalid
+ * (the TpProxy::invalidated signal).
+ *
+ * The returned handle is not necessarily valid forever (the
+ * notify::self-handle signal will be emitted if it changes, which can happen
+ * on protocols such as IRC). Construct a #TpContact object if you want to
+ * track the local user's identifier in the protocol, or other information
+ * like their presence status, over time.
+ *
+ * Returns: the value of the TpConnection:self-handle property
+ *
+ * Since: 0.7.26
+ */
+TpHandle
+tp_connection_get_self_handle (TpConnection *self)
+{
+  g_return_val_if_fail (TP_IS_CONNECTION (self), 0);
+  return self->priv->self_handle;
+}
+
 /**
  * tp_connection_get_status:
  * @self: a connection
@@ -822,18 +1042,19 @@ raise_invalidated:
 
 /**
  * TpConnectionNameListCb:
- * @names: %NULL-terminated array of connection bus names,
+ * @names: %NULL-terminated array of @n connection bus names,
  *   or %NULL on error
  * @n: number of names (not including the final %NULL), or 0 on error
- * @cms: %NULL-terminated array of connection manager names in the same order
- *   as @names, or %NULL on error
- * @protocols: %NULL-terminated array of protocol names in the same order
- *   as @names, or %NULL on error
+ * @cms: %NULL-terminated array of @n connection manager names
+ *   (e.g. "gabble") in the same order as @names, or %NULL on error
+ * @protocols: %NULL-terminated array of @n protocol names as defined in the
+ *   Telepathy spec (e.g. "jabber") in the same order as @names, or %NULL on
+ *   error
  * @error: %NULL on success, or an error that occurred
  * @user_data: user-supplied data
  * @weak_object: user-supplied weakly referenced object
  *
- * Signature of the callback supplied to tp_list_connection_managers().
+ * Signature of the callback supplied to tp_list_connection_names().
  *
  * Since: 0.7.1
  */
@@ -967,10 +1188,14 @@ list_context_free (gpointer p)
  *   @callback, but also if the D-Bus connection fails or if the @weak_object
  *   goes away
  * @weak_object: if not %NULL, will be weakly referenced; the callback will
- *   not be called, and the call will be cancelled, if the object has vanished
+ *   not be called if the object has vanished
  *
- * List the available (running or installed) connection managers. Call the
- * callback when done.
+ * List the bus names of all the connections that currently exist, together
+ * with the connection manager name and the protocol name for each connection.
+ * Call the callback when done.
+ *
+ * The bus names passed to the callback can be used to construct #TpConnection
+ * objects for any connections that are of interest.
  *
  * Since: 0.7.1
  */
