@@ -76,6 +76,8 @@ TpConnection *tp_connection_new (TpDBusDaemon *dbus, const gchar *bus_name,
 TpConnectionStatus tp_connection_get_status (TpConnection *self,
     TpConnectionStatusReason *reason);
 
+TpHandle tp_connection_get_self_handle (TpConnection *self);
+
 gboolean tp_connection_is_ready (TpConnection *self);
 
 gboolean tp_connection_run_until_ready (TpConnection *self,
@@ -131,4 +133,17 @@ G_END_DECLS
 
 #include <telepathy-glib/_gen/tp-cli-connection.h>
 
+G_BEGIN_DECLS
+
+/* connection-handles.c again - this has to come after the auto-generated
+ * stuff because it uses an auto-generated typedef */
+
+void tp_connection_get_contact_attributes (TpConnection *self,
+    gint timeout_ms, guint n_handles, const TpHandle *handles,
+    const gchar * const *interfaces, gboolean hold,
+    tp_cli_connection_interface_contacts_callback_for_get_contact_attributes callback,
+    gpointer user_data, GDestroyNotify destroy, GObject *weak_object);
+
+G_END_DECLS
+
 #endif
