@@ -894,7 +894,7 @@ contacts_context_fail (ContactsContext *c,
  * valid (it may be empty), and @failed_id_errors will map the IDs
  * that were not valid to a corresponding #GError (if the connection manager
  * complies with the Telepathy spec, it will have domain %TP_ERRORS and code
- * %TP_ERROR_NOT_AVAILABLE).
+ * %TP_ERROR_INVALID_HANDLE).
  *
  * If an unrecoverable error occurs (for instance, if @connection
  * becomes disconnected) the whole operation fails, and no contacts
@@ -1773,6 +1773,14 @@ contacts_get_attributes (ContactsContext *context)
   const gchar **supported_interfaces;
   guint i;
 
+  /* tp_connection_get_contact_attributes insists that you have at least one
+   * handle; skip it if we don't (can only happen if we started from IDs) */
+  if (context->handles->len == 0)
+    {
+      contacts_context_continue (context);
+      return;
+    }
+
   g_assert (tp_proxy_has_interface_by_id (context->connection,
         TP_IFACE_QUARK_CONNECTION_INTERFACE_CONTACTS));
   g_assert (contact_attribute_interfaces != NULL);
@@ -2042,7 +2050,8 @@ contacts_requested_one_handle (TpConnection *connection,
       c->next_index++;
     }
   else if (error->domain == TP_ERRORS &&
-      (error->code == TP_ERROR_NOT_AVAILABLE ||
+      (error->code == TP_ERROR_INVALID_HANDLE ||
+       error->code == TP_ERROR_NOT_AVAILABLE ||
        error->code == TP_ERROR_INVALID_ARGUMENT))
     {
       g_hash_table_insert (c->request_errors,
@@ -2105,7 +2114,8 @@ contacts_requested_handles (TpConnection *connection,
         }
     }
   else if (error->domain == TP_ERRORS &&
-      (error->code == TP_ERROR_NOT_AVAILABLE ||
+      (error->code == TP_ERROR_INVALID_HANDLE ||
+       error->code == TP_ERROR_NOT_AVAILABLE ||
        error->code == TP_ERROR_INVALID_ARGUMENT))
     {
       /* One of the strings is bad. We don't know which, so split them. */
