@@ -204,6 +204,9 @@ gboolean tp_group_mixin_change_members (GObject *obj,
     const gchar *message, TpIntSet *add, TpIntSet *del,
     TpIntSet *add_local_pending, TpIntSet *add_remote_pending, TpHandle actor,
     TpChannelGroupChangeReason reason);
+gboolean tp_group_mixin_change_members_detailed (GObject *obj, TpIntSet *add,
+    TpIntSet *del, TpIntSet *add_local_pending, TpIntSet *add_remote_pending,
+    const GHashTable *details);
 void tp_group_mixin_change_self_handle (GObject *obj,
     TpHandle new_self_handle);
 
